"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Connection_1 = __importDefault(require("../Connection"));
const schema_1 = __importDefault(require("./schema"));
const checkConnection = {
    preHandler: function (request, reply, done) {
        const { name } = request.params;
        if (!connections.has(name))
            return reply.sendError(404, `Connection ${name} not found!`);
        done();
    }
};
exports.default = async (app) => {
    app.decorateRequest('getConnection', function () {
        return connections.get(this.params.name);
    });
    app.get('/', (request, reply) => {
        reply.send(connections.map(connection => connection.toObject()));
    });
    app.post('/', {
        schema: {
            body: schema_1.default.connection
        }
    }, async (request, reply) => {
        if (connections.has(request.body.name))
            return reply.sendError(409, `Connection already exists!`);
        const connection = await new Connection_1.default(request.body.name).save();
        if (request.body.endpoints && request.body.endpoints.length) {
            for (let endpoint of request.body.endpoints) {
                await connection.addEndpoint(endpoint);
            }
        }
        reply.send(connection.toObject());
    });
    app.get('/:name', checkConnection, async (request, reply) => {
        const connection = request.getConnection();
        reply.send(connection.toObject());
    });
    app.post('/:name/disable', checkConnection, async (request, reply) => {
        const connection = request.getConnection();
        if (connection.disabled) {
            return reply.sendError(400, `Connection ${connection.name} is already disabled!`);
        }
        await connection.disable().save();
        reply.send(connection.toObject());
    });
    app.post('/:name/enable', checkConnection, async (request, reply) => {
        const connection = request.getConnection();
        if (!connection.disabled) {
            return reply.sendError(400, `Connection ${connection.name} is already enabled!`);
        }
        await connection.enable().save();
        reply.send(connection.toObject());
    });
    app.delete('/:name', checkConnection, async (request, reply) => {
        const connection = request.getConnection();
        await connection.delete();
        return reply.send(connection.toObject());
    });
    app.get('/:name/endpoints', checkConnection, async (request, reply) => {
        const connection = request.getConnection();
        reply.send(connection.toObject().endpoints);
    });
    app.post('/:name/endpoints', {
        schema: {
            body: schema_1.default.endpoint
        },
        ...checkConnection
    }, async (request, reply) => {
        const connection = request.getConnection();
        if (connection.hasEndpoint(request.body.id)) {
            return reply.sendError(400, `Connection ${connection.name} already has endpoint ${request.body.id}`);
        }
        await connection.addEndpoint(request.body);
        reply.send(connection.toObject().endpoints);
    });
    app.delete('/:name/endpoints/:id', checkConnection, async (request, reply) => {
        const connection = request.getConnection();
        const { id, name } = request.params;
        if (!connection.has(id))
            return reply.sendError(404, `Connection ${name} doesn't have an endpoint ${id}!`);
        await connection.removeEndpoint(id).save();
        reply.send(connection.toObject().endpoints);
    });
};
